%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Project     : PON_HD
% Module      : SymbolEstimation_MMSE
% Description : Symbol Estimation. Use MMSE detector.
%               y    = Hs + n 
%               Ryy  = H * H' + N_0 * I
%               W    = Ryy ^(-1) * H
%               sEst = W' * y
%
% References  : 
%   [R1] Proakis. Digital Communications 5th ed.
%   [R2] David Tse. Fundamentals of wireless communication 5th ed.
%
%Version|Date    |author|Description
%-------|--------|------|-------------------------------------------------------
%01.01  |20201111|L,YM  |Initial version
%-------|--------|------|-------------------------------------------------------
%01.01  |20201113|L,YM  |Decode by H column power from max to min
%-------|--------|------|-------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Dsp = SymbolEstimation_MMSE(PARAM,Dsp)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% input isolation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
H      = Dsp.RxHEst;
SNR    = Dsp.RxSnrEstAvg;
DataIn = Dsp.RxSymbol;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% coeffients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[DataLength,DataWidth] = size(DataIn);

%normalize H and DataIn
NormCoef = sqrt( PARAM.Nt * DataWidth / sum(abs(H).^2, 'all') );

H      = H      .* NormCoef;
DataIn = DataIn .* NormCoef;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% main function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% truncate incomplete frame
yAxis = DataIn(1:DataLength,:); %truncate incomplete frame

% generate MMSE Matrix W
Ryy   = H * H' + 1/SNR * eye(DataWidth);
W = Ryy^(-1) * H;

% estimate s
DataOut = (W' * yAxis.').';
Dsp.RxSymbol = DataOut;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% plot
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% if PARAM.PlotOn
%   %% constellation
%   %figure settings
%   figure;
%   set(gcf,'unit','inches','position',...
%     [PARAM.PlotMinWidth  + PARAM.PlotWidth  * 3,...
%      PARAM.PlotMinHeight + PARAM.PlotHeight * 2,...
%      PARAM.PlotWidth,...
%      PARAM.PlotHeight]);%size of figure
%   set(gca,'LooseInset',get(gca,'TightInset'));%clear outer white
%   
%   %plot complete frame only.
%   FrameFlag = DspRx.FrameFlag;
%   FrameNum  = floor( (size(DataIn, 1) - FrameFlag + 1) / PARAM.FrameLength);
%   FrameTail = FrameFlag - 1 + FrameNum * PARAM.FrameLength;
%   DataPlot = DataOut(FrameFlag : FrameTail, :);
%   
%   %plot function
%   for i1 = 1 : PARAM.Nt
%     ax = subplot(PARAM.PlotRowTx , PARAM.PlotColTx , i1);
%     
%     if PARAM.ScatterColorOn
% %       colormap hot
%       ScatterColormap(ax,real(DataPlot(:,i1)), imag(DataPlot(:,i1)));
% %       colorbar
%     else
%       plot(ax, DataPlot(:,i1), 'g.'); hold on;
%     end
%         
%     %% plot FormatMap
%     plot(ax, PARAM.FormatMap, 'r+' , 'LineWidth' , 2); hold on;
% 
%     grid on; grid minor;
%     axis equal
%     title(['MMSE Tx',num2str(i1)])
%   end
% end

end%function